@@ -8,10 +8,10 @@ use parent qw(
     Crypt::Perl::KeyBase
 );
 
-use Crypt::Format ();
 use Module::Load ();
 
 use Crypt::Perl::BigInt ();
+use Crypt::Perl::X ();
 
 BEGIN {
     __PACKAGE__->mk_ro_accessors('modulus');
@@ -36,6 +36,7 @@ sub new {
 sub to_pem {
     my ($self) = @_;
 
+    require Crypt::Format;
     return Crypt::Format::der2pem( $self->to_der(), $self->_PEM_HEADER() );
 }
 
@@ -84,6 +85,15 @@ sub to_der {
     return $self->_to_der($self->_ASN1_MACRO());
 }
 
+sub algorithm_identifier {
+    my ($self) = @_;
+
+    return {
+        algorithm => OID_rsaEncryption(),
+        parameters => Crypt::Perl::ASN1::NULL(),
+    };
+}
+
 use constant OID_rsaEncryption => '1.2.840.113549.1.1.1';
 
 sub _to_subject_public_der {
@@ -92,10 +102,7 @@ sub _to_subject_public_der {
     my $asn1 = $self->_asn1_find('SubjectPublicKeyInfo');
 
     return $asn1->encode( {
-        algorithm => {
-            algorithm => OID_rsaEncryption(),
-            parameters => Crypt::Perl::ASN1::NULL(),
-        },
+        algorithm => $self->algorithm_identifier(),
         subjectPublicKey => $self->_to_der('RSAPublicKey'),
     } );
 }
@@ -103,7 +110,7 @@ sub _to_subject_public_der {
 sub get_struct_for_public_jwk {
     my ($self) = @_;
 
-    Module::Load::load('MIME::Base64');
+    require MIME::Base64;
 
     return {
         kty => 'RSA',
@@ -117,13 +124,13 @@ sub get_struct_for_public_jwk {
 sub _asn1_find {
     my ($self, $macro) = @_;
 
-    Module::Load::load('Crypt::Perl::ASN1');
-    Module::Load::load('Crypt::Perl::RSA::Template');
+    require Crypt::Perl::ASN1;
+    require Crypt::Perl::RSA::Template;
     my $asn1 = Crypt::Perl::ASN1->new()->prepare(
         Crypt::Perl::RSA::Template::get_template('INTEGER'),
     );
 
-    $asn1->find($macro);
+    return $asn1->find($macro);
 }
 
 sub _to_der {
@@ -153,14 +160,15 @@ sub _verify {
     if ($scheme eq 'PKCS1_v1_5') {
         my $key_bytes_length = $self->modulus_byte_length();
         if (length($octets) != $key_bytes_length) {
-            die sprintf( "Invalid PKCS1_v1_5 length: %d (should be %d)", length($octets), $key_bytes_length );
+            my $err = sprintf( "Invalid PKCS1_v1_5 length: %d (should be %d)", length($octets), $key_bytes_length );
+            die Crypt::Perl::X::create('Generic', $err);
         }
 
-        Module::Load::load('Crypt::Perl::RSA::PKCS1_v1_5');
+        require Crypt::Perl::RSA::PKCS1_v1_5;
         return $digest eq Crypt::Perl::RSA::PKCS1_v1_5::decode($octets, $hasher);
     }
 
-    die "Unknown scheme: “$scheme”";
+    die Crypt::Perl::X::create('Generic', "Unknown signature scheme: “$scheme”");
 }
 
 1;
